import * as THREE from 'three';
import { EventDispatcher } from 'three';
import { GLTFLoader } from 'three/examples/jsm/loaders/GLTFLoader';
import { DRACOLoader } from 'three/examples/jsm/loaders/DRACOLoader';
import { TransformControls } from 'three/examples/jsm/controls/TransformControls';
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls';



const imageSize = { x: 400, y: 300 };
const imageHalfWidth = imageSize.x / 2;
const imageHalfHeight = imageSize.y / 2;
const imageResolution = 75;
export class SceneHandler extends EventDispatcher {
    static raycaster = new THREE.Raycaster();
    static scene = undefined;
    static camera = undefined;
    static renderer = undefined;
    static finalVectorArray = [];
    static mixerList = [];
    static animationsList = [];
    static modelsList = new THREE.Group();
    static gltfLoader = new GLTFLoader().setDRACOLoader(new DRACOLoader().setDecoderPath('../js/draco/gltf/'));
    static textureLoader = new THREE.TextureLoader();
    static clock = new THREE.Clock();
    static transforControl = undefined;
    static currentBlindModel = undefined;
    static zoomvalue = 0.1;
    static defaultZoom = undefined;
    static orbitControl = undefined;
    static listeners = [];
    static configObj = {
        "foldIn" : false,
        "foldOut" : false,
        "zoomIn" : true,
        "zoomOut" : true,
        "open" : true,
        "close" : true,
        
    }
  
    static removeModelList =()=>
    {
        SceneHandler.scene.remove(SceneHandler.modelsList);
    }

    static setupScene(three) {
        this.scene = three.scene;
        this.camera = three.camera;
        this.renderer = three.gl;
        this.renderer.shadowMap.enabled = true; // Enable shadows
        this.renderer.shadowMap.type = THREE.PCFSoftShadowMap;
        this.raycaster = three.raycaster;
        this.scene.add(this.modelsList);
        this.defaultZoom = this.camera.position.clone().z;
        this.orbitControl = new OrbitControls(this.camera,this.renderer.domElement);
        this.orbitControl.enableRotate = false;
        this.orbitControl.minDistance = 0.8;
        this.orbitControl.maxDistance = 5;
        this.orbitControl.update();
        SceneHandler.animate();

    }

    static addEventListener(type, callback) {
        this.listeners.push({ type, callback });
    }

    static dispatchEvent(event) {
        this.listeners.forEach(listener => {
            if (listener.type === event.type) {
                listener.callback(event);
            }
        });
    }

    static setBaseModel()
    {

    }

    static addLight()
    {

        console.log('add liteeeee...')
        const directionalLight = new THREE.DirectionalLight(0xffffff, 0.2);
        directionalLight.position.set(0, 3, -2);
        directionalLight.castShadow = true;
        directionalLight.shadow.camera.near = 0.1; 
        directionalLight.shadow.camera.far = 30;
        directionalLight.target.position.set(0, 0, 0);
        SceneHandler.scene.add(directionalLight);
    }

    static addShadowPlane()
    {
        console.log('add addShadowPlane df...')
        const planeGeometry = new THREE.PlaneGeometry(5,2);
        const loader = new THREE.TextureLoader();
      
        // SceneHandler.planeMaterial = new THREE.ShadowMaterial({ opacity: 0.2 });
        SceneHandler.planeMaterial =  new THREE.MeshPhongMaterial({ color:0x00ff00 });
       
        SceneHandler.planeMaterial.side = THREE.DoubleSide;
        const plane = new THREE.Mesh(planeGeometry, SceneHandler.planeMaterial);
        plane.rotateX(-1.07);
        plane.position.set( 0,-1.2913098317295773,0.6473203737076866) ;
        plane.scale.y = 0.5;
        plane.scale.x = 0.8;
        plane.receiveShadow = true;
        SceneHandler.scene.add(plane);
    }


    static fetchData = async (image) => {
        console.log(image, 'imageimageimageimageimageimageimage');
        try {
            const blob = await fetch(image).then((response) => response.blob());
            const reader = new FileReader();
            reader.onload = function (event) {
                const base64Image = event.target.result.split(',')[1];

                const apiUrl = 'https://mukesh3444-ai-detect.hf.space/?logs=container';
                const payload = {
                    img: base64Image,
                };
                const jsonPayload = JSON.stringify(payload);
                const headers = {
                    'Content-Type': 'application/json',
                };
                const requestOptions = {
                    method: 'POST',
                    headers: headers,
                    body: jsonPayload,
                };

                fetch(apiUrl, requestOptions, {
                    mode: 'no-cors',
                })
                    .then((response) => {
                        if (!response.ok) {
                            throw new Error(`HTTP Error! Status: ${response.status}`);
                        }
                        return response.json();
                    })
                    .then((data) => {
                        SceneHandler.createBlindsData(data.windows_vertices)
                       
                    })
                    .catch((error) => {
                        console.error('Error:', error);
                    });
            };
            reader.readAsDataURL(blob);
        } catch (error) {
            console.error('Error fetching or processing API data:', error);
        }
    }

    static sortXYPointsForRectangle = (points) => {
        let centerX = 0;
        let centerY = 0;
        for (const [x, y] of points) {
            centerX += parseInt(x);
            centerY += parseInt(y);
        }
        centerX /= points.length;
        centerY /= points.length;

        points.sort(([x1, y1], [x2, y2]) => {
            const angle1 = Math.atan2(y1 - centerY, x1 - centerX);
            const angle2 = Math.atan2(y2 - centerY, x2 - centerX);
            return angle1 - angle2;
        });
    }

    static createBlindsData = (apiData) => {
        SceneHandler.finalVectorArray = [];
        let scale = 0.02;
        let arrayPixelPoints = JSON.parse(JSON.stringify(apiData));
        const tempArray = [];
        for (let i = 0; i < arrayPixelPoints.length; i++) {
            let arrayvector = [];
            SceneHandler.sortXYPointsForRectangle(arrayPixelPoints[i]);
            arrayPixelPoints[i].push(arrayPixelPoints[i][0]);

            arrayPixelPoints[i].forEach((item) => {
                let vector = new THREE.Vector3(
                    (parseInt(item[0]) - imageHalfWidth) / imageResolution,
                    -(parseInt(item[1]) - imageHalfHeight) / imageResolution,
                    0.05
                );
                arrayvector.push(vector);
            });

            tempArray.push([arrayvector[0], arrayvector[2], scale]);
            // console.log(arrayvector[0], arrayvector[2], scale);
        }

        SceneHandler.finalVectorArray = tempArray;
        console.log(SceneHandler.finalVectorArray);
        SceneHandler.createBlinds();
       
    };
    

    
    static playAnimation(type,status)
    {
        
        this.animationsList.forEach(item=>
        {
            let aniName=  item.name.toLowerCase();
            if(status && !aniName.includes("unfold"))
            {
                if(!item.paused && aniName.includes(type))
                {
                    item.timeScale = 1;
                    item.play();
                    setTimeout(() => {
                        item.paused = true;
                    }, 1800);
                }

                else if(item.paused && aniName.includes(type))
                {
                    item.timeScale = 1;
                    item.paused = false;
                    setTimeout(() => {
                        item.paused = true;
                    }, 500);
                }

                
            }

            else if(!status && !aniName.includes("unfold"))
            {
                item.timeScale = -1;
                if(!item.paused && aniName.includes(type))
                {
                    item.play();
                    setTimeout(() => {
                        
                        item.paused = true;
                    }, 1800);
                }

                else if(item.paused && aniName.includes(type))
                {
                    item.paused = false;
                    setTimeout(() => {
                        item.paused = true;
                    }, 500);
                }

                
            }
        })
    }

    static changeTexture(image)
    {

        const newTexture = SceneHandler.textureLoader.load(image);
        SceneHandler.modelsList.traverse(item=>
            {
                if(item.isMesh)
                {
                    if(item.name.includes("Fabric") || item.name.includes("Wood") || item.name.includes("wood"))
                    {
                        item.material.color = null;
                        item.material.map = newTexture;
                        item.material.needsUpdate = true;
                    }
                }
            });
    }

    static animate() {
        requestAnimationFrame(SceneHandler.animate);
        if( SceneHandler.mixerList.length > 0)
        {
            let delta = SceneHandler.clock.getDelta();
            SceneHandler.mixerList.forEach((mixer) => {
                mixer.update(delta);
            });
        }
       
    }

    static changeSampleRoom(image)
    {
            SceneHandler.fetchData(image);
            const backgroundTexture = SceneHandler.textureLoader.load(image);
            let bgmesh = SceneHandler.scene.getObjectByName("bg");
            if(!bgmesh) return;
            bgmesh.material.map = backgroundTexture;
            bgmesh.material.needsUpdate = true;
    }

    static updateBlinds()
    {
        if(SceneHandler.modelsList.children.length > 0 )
        {

            SceneHandler.modelsList.children.map((item, index)=>
            {
                let pointObj = this.finalVectorArray[index];
                let point = pointObj[0];
                let lastPt = pointObj[1];
                let scale = pointObj[2];

                // console.log(pointObj,item);

                const boundingBox = new THREE.Box3().setFromObject(item);
                const size = new THREE.Vector3();
                boundingBox.getSize(size);
                let actualWidth = lastPt.x - point.x;
                let actualHeight = lastPt.y - point.y;
        
                item.scale.set(actualWidth / size.x, Math.abs(actualHeight / size.y), scale);
                item.position.set((point.x + ((lastPt.x - point.x) / 2))+0.05, lastPt.y, point.z+0.08);
                if(index == 0) item.position.x = item.position.x - 0.1;
            });
        }
    }

    static resetZoom()
    {
        SceneHandler.camera.position.z = SceneHandler.defaultZoom;
        SceneHandler.orbitControl.update();
    }

    static zoom(isZoomIn)
    {
        if(isZoomIn)
       {
            if(SceneHandler.camera.position.z > 0.8)
            {
                SceneHandler.camera.position.z -= SceneHandler.zoomvalue;
            }
       }
       else
       {
            if(SceneHandler.camera.position.z <= SceneHandler.defaultZoom+2)
            {
                SceneHandler.camera.position.z += SceneHandler.zoomvalue;
            }
       }
       
    }
    
    static emptyGroup() {
        while (SceneHandler.modelsList.children.length > 0) {
          let child = SceneHandler.modelsList.children[0];
          SceneHandler.modelsList.remove(child);
        }
      }

    static deleteAllBlindDetails()
    {
        SceneHandler.emptyGroup();
        SceneHandler.animationsList = [];
        SceneHandler.mixerList = [];
    }

    static changeBlind(glbModel)
    {
            SceneHandler.currentBlindModel = glbModel;
            SceneHandler.createBlinds();
    }

    static loadModel(item,index) {
        return new Promise((resolve, reject) => {
            let windowScene = undefined;
            let mixer = undefined;
            let animations = undefined;
    
            this.gltfLoader.load(SceneHandler.currentBlindModel, function (gltf) {
                let point = item[0];
                    let lastPt = item[1];
                    let scale = item[2];

                    windowScene = gltf.scene;
                    console.log(windowScene);
                    window.scene = SceneHandler.scene;
                    windowScene.traverse(item=>
                        {
                            if(item.isMesh)
                            {
                                item.castShadow = true;
                                item.receiveShadow = true;
                            }
                        })
                    windowScene.castShadow = true;
                    windowScene.receiveShadow = true;
                    animations = gltf.animations;

                    if(animations.length > 0)
                    {
                        SceneHandler.configObj["foldIn"] = false;
                        SceneHandler.configObj["foldOut"] = false;
                        animations.map(anime=>
                        {
                            if(anime.name.includes("Fold") || anime.name.includes("fold"))
                            {
                                SceneHandler.configObj["foldIn"] = true;
                                SceneHandler.configObj["foldOut"] = true;
                            }
                            
                        })
                    }
                    mixer = new THREE.AnimationMixer(gltf.scene);
                    SceneHandler.mixerList.push(mixer);

                    const boundingBox = new THREE.Box3().setFromObject(windowScene);
                    const size = new THREE.Vector3();
                    boundingBox.getSize(size);
                    let actualWidth = lastPt.x - point.x;
                    let actualHeight = lastPt.y - point.y;
                    windowScene.scale.set(actualWidth / size.x, Math.abs(actualHeight / size.y), scale);
                    windowScene.position.set((point.x + ((lastPt.x - point.x) / 2))+0.05, lastPt.y, point.z+0.08);
                    if(index == 0) windowScene.position.x = windowScene.position.x - 0.1;
                    if(index == 1) windowScene.position.x = windowScene.position.x + 0.08;
                    if(animations.length > 0)
                    {
                        animations.forEach(clip=>
                        {
                            let action = mixer.clipAction(clip);
                            action.name = clip.name;
                            action.loop = THREE.LoopOnce;
                            action.clampWhenFinished = true;
                            SceneHandler.animationsList.push(action);
                        })
                       
                    }
                    SceneHandler.modelsList.add(windowScene);
                    resolve(); 
            });
        });
    }


    static createBlinds()
    {
      
        SceneHandler.deleteAllBlindDetails();
        Promise.all(this.finalVectorArray.map((item, index) => this.loadModel(item,index)))
        .then(() => {
            SceneHandler.dispatchEvent({ type: "loadedData", data: true });
            SceneHandler.dispatchEvent({ type: "uiOptions", data: SceneHandler.configObj });
        })
        .catch(error => {
            console.error('Error loading models:', error);
        });
    }

}